extends Node
# このファイルはスクリプト間での参照を自由にするため、Autoloadに指定します

### プロジェクト毎に個別設定をする項目 ###
# 著作者（自由に変更してください）
const copyright_name = "©Developer, Version "
# ソフトウェアバージョン
const app_version = "2.0.0"
# デバッグモードの切り替え
const enable_storylib_debug_mode = false
# セーブの暗号化を使用する(trueを推奨)
# ※注意: この設定を変更すると、既存のセーブデータが正しく認識されなくなります
const use_encrypt : = true
# ※注意: 以下の2つの変数をプロジェクト途中で変更すると、既存のセーブデータが正しく認識されなくなります
# 複合化キー（プロジェクト開始時に設定してください）
const encrypt_key = "encrypt123456789"
# データ引継ぎ用のシステムキー
const transfer_system_code = "SAVE"
####################################

# Signals - Main
signal node
signal set_image
signal audio
signal manage_save_data
signal get_preloaded_instance
signal has_temporary_slot_data
signal update_file
signal restart_for_transfer # Start
# Signal - Start
signal initialize_start
signal run_load_data_from_start_scene
signal run_open_saves_panel_delete
# Signal - Story
signal set_jump_marker
# Signal - ChoicesPanel
signal set_choise_buttons
# Signal - SavesPanel
signal set_save_slot_buttons

const assets_path: String = "res://SystemAssets/"
# 最初にプリロードする以外のシーンのパス
var scenes = {
	"main": assets_path + "Scenes/Main.tscn",
}
# 最初にプリロードする以外のノードのパス（シーンよりも小規模なものにお使いください）
var nodes = {
	"choice_button": assets_path + "Nodes/ChoiceButton.tscn",
	"save_slot_button": assets_path + "Nodes/SaveSlotButton.tscn"
}

# ゲーム内テキストデータの参照先
const events_directory = "res://Events"
# 現在セットされているStoryLibのテキスト
var current_storylib_text = "opening"
# 現在のロケーション名
var current_location = ""
# StoryLibの選択肢とジャンプマーカー
var storylib_choices = {}

# ローカルセーブファイルのルートディレクトリ
const root_directory = "user://"
# ローカルストレージのセーブファイル名
const save_file_name = "saves"
# セーブデータの拡張子
const save_file_extension = "json"
# セーブ可能なスロット数
const max_save_slot = 5
# セーブ変数を扱うためのパス
const data_templates_directory: String = "res://DataTemplates/"
const per_slot_data_file_name: String = "PerSlotData"
# アプリ起動中に使われる一時セーブの大枠
var temporary_saves = {}
# プレイ中に使われるセーブスロットデータ
var current_temporary_slot_data = {}
# プレイ中のセーブスロット（セーブデータを正常に認識させるため、初期設定の0のままにしておいてください）
var current_save_slot = "0"

# 制御文字のリストのバス
const control_char_file_name: String = "ControlCharacters"

# 指定した内部ファイルのデータを全て返す
func get_file_text(file_path, is_encrypted):
	var file_data
	# ファイルが存在している場合
	if FileAccess.file_exists(file_path):
		var file_access
		# 暗号化判定
		if is_encrypted:
			file_access = FileAccess.open_encrypted_with_pass(file_path, FileAccess.READ, Data.encrypt_key)
		else:
			file_access = FileAccess.open(file_path, FileAccess.READ)
		# ファイルが正常に開かれた場合
		if file_access:
			match file_path.get_extension():
				"json":
					# 文字列を正常に認識させるためにstrip_edges()で改行等の余白を削除
					file_data = JSON.parse_string(file_access.get_as_text().strip_edges())
				"txt":
					file_data = file_access.get_as_text()
			
			file_access.close()
		else:
			push_error("Error get_file_text: ファイルデータの参照に失敗しました。", self)
	
	return file_data
