extends Node

@onready var copy_option: Control = $Copy
@onready var transfer_option: Control = $Transfer
@onready var pdescription1: Label = $Description
@onready var pdescription2: Label = $Description2
@onready var confirm: ColorRect = $Confirm
@onready var confirm_info: Label = $Confirm/Info
@onready var mask: ColorRect = $Mask
# 現在のパネルのモード
var panel_mode = "none"

func _ready():
	confirm.hide()
	# ローカルにセーブデータが存在している場合
	if FileAccess.file_exists(Data.root_directory + "/" + Data.save_file_name + "." + Data.save_file_extension):
		transfer_option.set_position(Vector2(0, 860))
		copy_option.show()
	else:
		copy_option.hide()
		transfer_option.set_position(Vector2(0, 300))
	
	transfer_option.show()
	mask.hide()

func on_button_pressed(task):
	mask.show()
	
	match task:
		"copy_saves":
			# 一時セーブが空ではない場合
			if Data.temporary_saves != null and Data.temporary_saves != {}:
				# 暗号化された一時セーブデータをクリップボードにコピーする
				DisplayServer.clipboard_set(Data.transfer_system_code + str(Marshalls.variant_to_base64(Data.temporary_saves)))
				confirm_info.set_text("現在のセーブデータを\nブラウザのローカルストレージに\nコピーしました")
				confirm.show()
		"transfer_saves":
			var copied_saves = DisplayServer.clipboard_get()
			#クリップボードにコピーされたセーブデータのシステムコードを照合し、一致した場合
			if copied_saves.left(Data.transfer_system_code.length()) == Data.transfer_system_code:
				# システムコードをトリムする
				copied_saves = copied_saves.substr(Data.transfer_system_code.length())
				var conv_saves = JSON.parse_string(str(Marshalls.base64_to_variant(copied_saves)))
				# 各データの照合を行う
				var template_path = Data.data_templates_directory + Data.per_slot_data_file_name + ".json"
				# テンプレートデータが存在している場合
				if FileAccess.file_exists(template_path):
					var file_access
					var template_data
					file_access = FileAccess.open(template_path, FileAccess.READ)
					# ファイルが正常に開かれた場合
					if file_access:
						# 文字列を正常に認識させるためにstrip_edges()で改行等の余白を削除
						template_data = JSON.parse_string(file_access.get_as_text().strip_edges())
						file_access.close()
						# テンプレートデータと引継ぎデータのキーを照合する
						var keys_match = true
						# セーブ可能なスロット数の範囲内に収まっている場合
						if conv_saves.size() <= Data.max_save_slot:
							# 引継ぎデータに保存されているスロットと同じ数だけ繰り返す
							for top_key in conv_saves.keys():
								# 一つずつキーを比較し、不一致があればfalseを返す
								for key in conv_saves[top_key]:
									if not template_data.has(key):
										keys_match = false
							# キーが全て一致したらデータの引継ぎを開始する
							if keys_match:
								Data.emit_signal("update_file", Data.root_directory, Data.save_file_name, Data.save_file_extension, conv_saves, Data.use_encrypt)
								confirm_info.set_text("セーブデータの引継ぎに\n成功しました！")
								confirm.show()
								panel_mode = "transfer"
					else:
						push_error("Error _get_file_text: ファイルの参照に失敗しました。", self)
				else:
					confirm_info.set_text("【システムテンプレートエラー】\n処理を中止します")
					confirm.show()
			else:
				confirm_info.set_text("クリップボードに\n引継ぎ可能なセーブデータが\n確認できませんでした")
				confirm.show()
		"hide_confirm":
			if panel_mode == "transfer":
				Data.emit_signal("restart_for_transfer")
				queue_free()
			else:
				confirm.hide()
		"close":
			queue_free()
	
	mask.hide()
