extends Control
# セーブデータ選択画面
@onready var title_label: Label = $TitleLabel
@onready var confirm: ColorRect = $Confirm
@onready var confirm_info: Node = $Confirm/Info
# 現在のパネルのモード
var panel_mode = "load"
# 現在選択中のスロット
var current_slot

func _ready():
	Data.connect("set_save_slot_buttons", Callable(self, "_set_save_slot_buttons"))
	confirm.hide()

func _set_save_slot_buttons(current_panel_mode):
	# ボタンのポジションを正確に反映させるため、コンテナの前にControlノードを配置しています
	var control_node = $ScrollContainer/Control
	var container_node = $ScrollContainer/Control/VBoxContainer
	var button_position = Vector2(0, 0)
	var buttons_size_y = 0
	# ScrollContainerのサイズ
	var scroll_container_size = $ScrollContainer.get_size()
	# ボタンの間隔
	var button_spacing = 50
	# パネルモードを指定する
	panel_mode = current_panel_mode
	
	match current_panel_mode:
		"save":
			title_label.set_text("《データをセーブ》")
		"load":
			title_label.set_text("《データをロード》")
		"delete":
			title_label.set_text("《データを削除》")
	# 辞書に格納された最上位の階層の数だけ処理を繰り返す
	for i in Data.max_save_slot:
		# インスタンスを生成する
		var choice_button = load(Data.nodes["save_slot_button"]).instantiate()
		# ラベルノード
		var label_node = choice_button.get_node("Label")
		var save_number = str(int(i+1))
		# 一時データの中のセーブスロット情報を反映させる
		if str(i) in Data.temporary_saves:
			# ボタンのレイアウト
			choice_button.name = "Button " + save_number
			# locationキーが存在していて空ではない場合、それをセーブ名に反映させる
			if "location" in Data.temporary_saves[str(i)]:
				label_node.set_text(Data.temporary_saves[str(i)]["location_name"])
			else:
				label_node.set_text("セーブデータ " + save_number)
			
			choice_button.set_position(button_position)
			# ボタンが押された時に実行されるコールバックを設定
			choice_button.pressed.connect(_on_button_pressed.bind(i))
		else:
			choice_button.name = "Button " + save_number
			label_node.set_text("空のセーブスロット " + save_number)
			label_node.self_modulate = Color("#909090")
			# パネルモードがセーブの場合、全てのスロットを選べるようにする
			if panel_mode == "save":
				choice_button.pressed.connect(_on_button_pressed.bind(i))
		# 念のため可視化オンにしてから配置
		choice_button.show()
		container_node.add_child(choice_button)
		# 次のボタンの位置を調整
		button_position.y += choice_button.size.y + button_spacing
		# ボタン全体の縦サイズを加算
		buttons_size_y += choice_button.get_size().y + button_spacing
	# Containerのサイズを取得
	var container_size = container_node.get_rect().size
	# Containerの位置を調整
	var container_position = Vector2(
		(scroll_container_size.x - container_size.x) / 2,
		(scroll_container_size.y - buttons_size_y) / 2
	)
	# 選択肢が画面に収まりきらない場合、Controlの縦のサイズを調整してコンテナの縦位置を固定する
	if buttons_size_y > scroll_container_size.y:
		control_node.set_custom_minimum_size(Vector2(control_node.get_rect().size.x, buttons_size_y))
		container_position = Vector2((scroll_container_size.x - container_size.x) / 2, 50)
	# 配置
	container_node.set_position(container_position)

# ボタンが押された時のコールバック
func _on_button_pressed(button_index):
	current_slot = button_index
	
	match panel_mode:
		"save":
			confirm_info.set_text("スロット" + str(button_index + 1) + "\nに現在のデータをセーブしますか？")
		"load":
			confirm_info.set_text("スロット" + str(button_index + 1) + "\nをロードしますか？")
		"delete":
			confirm_info.set_text("スロット" + str(button_index + 1) + "\nのデータを削除しますか？")
	
	confirm.show()

func on_button_pressed(task):
	match task:
		"accept":
			match panel_mode:
				"save":
					Data.emit_signal("manage_save_data", "save_per_slot " + str(current_slot), Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
					# セーブ時に演出を加える際は、この部分に処理を追記してください。
					queue_free()
				"load":
					Data.emit_signal("manage_save_data", "load_per_slot " + str(current_slot), Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
					Data.current_storylib_text = Data.temporary_saves[str(current_slot).strip_edges()]["location"]
					Data.emit_signal("run_load_data_from_start_scene")
					queue_free()
				"delete":
					Data.emit_signal("manage_save_data", "delete_per_slot " + str(current_slot), Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
					Data.emit_signal("run_open_saves_panel_delete")
					queue_free()
		"hide_confirm":
			current_slot = null
			confirm.hide()
		"close":
			queue_free()
