extends Node
# スタート画面
@onready var debug_label: Label = $DebugLabel
@onready var copyright_label: Label = $CopyrightLabel
@onready var buttons: Control = $Buttons
@onready var button2: TextureButton = $Buttons/Button2
@onready var button3: TextureButton = $Buttons/Button3
@onready var button4: TextureButton = $Buttons/Button4
@onready var mask: ColorRect = $Mask
@onready var anim: AnimationPlayer = $AnimationPlayer

func _ready():
	# Data.gdを中継としたシグナルによって、各機能を他のスクリプトから呼び出せるようにする
	Data.connect("initialize_start", Callable(self, "_ready"))
	Data.connect("run_load_data_from_start_scene", Callable(self, "_run_load_data_from_start_scene"))
	Data.connect("run_open_saves_panel_delete", Callable(self, "_run_open_saves_panel_delete"))
	Data.connect("restart_for_transfer", Callable(self, "_restart_for_transfer"))
	# 各種表示の初期化
	mask.show()
	### debug mode
	if Data.enable_storylib_debug_mode:
		debug_label.show()
	else:
		debug_label.hide()
	# ローカルにセーブデータが存在している場合
	if FileAccess.file_exists(Data.root_directory + "/" + Data.save_file_name + "." + Data.save_file_extension):
		button2.show()
		button3.show()
	else:
		button2.hide()
		button3.hide()
	
	button4.show()
	copyright_label.set_text(Data.copyright_name + Data.app_version)
	Data.emit_signal("audio", "play", "bgm1", true)
	buttons.show()
	anim.play("fadein")

func _run_load_data_from_start_scene():
	on_button_pressed("load_data")

func _run_open_saves_panel_delete():
	# ローカルにセーブデータが存在している場合、セーブパネルを表示したままにする
	if FileAccess.file_exists(Data.root_directory + "/" + Data.save_file_name + "." + Data.save_file_extension):
		on_button_pressed("open_saves_panel_delete")
	else: # セーブファイルがない場合はスタート画面をリスタートさせる
		_ready()

func _restart_for_transfer():
	_ready()

func on_button_pressed(task):
	mask.show()
	
	match task:
		"new_play": # 最初のStoryを開く
			# 一時初期セーブをセット
			Data.emit_signal("has_temporary_slot_data", Data.data_templates_directory, Data.per_slot_data_file_name, true)
			# 次のシーンの準備
			Data.current_storylib_text = "opening"
			Data.emit_signal("audio", "stop", null, true)
			anim.play("fadeout")
			await get_tree().create_timer(1.0).timeout
			Data.emit_signal("node", "add", "StoryLib")
			queue_free()
		"load_data": # シグナル受信用: セーブデータのロード
			if Data.current_storylib_text != null and Data.current_storylib_text != "":
				Data.emit_signal("audio", "stop", null, true)
				anim.play("fadeout")
				await get_tree().create_timer(1.0).timeout
				Data.emit_signal("node", "add", "StoryLib")
				queue_free()
		"open_saves_panel_load": # ロード画面を開く
			Data.emit_signal("node", "add", "SavesPanel")
			Data.emit_signal("set_save_slot_buttons", "load")
			mask.hide()
		"open_saves_panel_delete": # データ削除画面を開く
			Data.emit_signal("node", "add", "SavesPanel")
			Data.emit_signal("set_save_slot_buttons", "delete")
			mask.hide()
		"open_save_transfer_panel": # 引継ぎメニューを開く
			Data.emit_signal("node", "add", "SaveTransferPanel")
			mask.hide()
		_:
			mask.hide()
			push_error("Error Start.gd: タスクが設定されていません。", self)
