extends Node
# StoryLibのコアファイルです
@onready var story_window: = $ImageWindow
@onready var bg_image: = $ImageWindow/BG
@onready var bg_mirror: = $ImageWindow/BGMirror
@onready var left_image: = $ImageWindow/Left
@onready var left_mirror: = $ImageWindow/LeftMirror
@onready var right_image: = $ImageWindow/Right
@onready var right_mirror: = $ImageWindow/RightMirror
@onready var center_image: = $ImageWindow/Center
@onready var center_mirror: = $ImageWindow/CenterMirror
@onready var front_image: = $ImageWindow/Front
@onready var front_mirror: = $ImageWindow/FrontMirror
@onready var left_emote: = $ImageWindow/LeftEmote
@onready var left_emote_mirror: = $ImageWindow/LeftEmoteMirror
@onready var right_emote: = $ImageWindow/RightEmote
@onready var right_emote_mirror: = $ImageWindow/RightEmoteMirror
@onready var center_emote: = $ImageWindow/CenterEmote
@onready var center_emote_mirror: = $ImageWindow/CenterEmoteMirror
@onready var fade_mask: = $ImageWindow/FadeMask

@onready var story_label: = $TextWindow/StoryLabel
@onready var page_arrow: = $TextWindow/PageArrow

@onready var input_window: = $InputWindow
@onready var next_button: = $InputWindow/NextButton
@onready var next_button_image: = $InputWindow/NextButton/Image

@onready var skip_button: = $SkipButton
@onready var credit_roll: = $CreditRoll
@onready var all_mask: = $AllMask
@onready var debug_menu: = $DebugMenu
@onready var debug_page_debugger: = $DebugMenu/PageDebugger
@onready var debug_line_edit: = $DebugMenu/PageDebugger/LineEdit

@onready var anim: = $AnimationPlayer
@onready var anim_left: = $AnimLeft
@onready var anim_right: = $AnimRight
@onready var anim_center: = $AnimCenter
@onready var anim_credit_roll: = $AnimCreditRoll

# テキストのスムーズ表示
var enable_text_animation_smoothing = true
# スキップボタンを有効にする
var enabled_skip_button = true
# スキップ用のジャンプマーカー
var skip_marker : String = ""
# 通常のジャンプマーカー
var jump_marker_is_valid: bool = false
var jump_marker: String = ""
# 行カウント
var line_number = 0
# 現在の発言者名
var current_speaker_name = ""
# リアルタイム更新用のテキスト
var realtime_texts = {}
var current_text = ""
# コマンド以外で[から始まる文字列を識別するための変数
var not_command = false
# タイマー系
const fadein_timer:= float(2.0)
const fadein_bgm_timer:= float(3.5)

func _ready():
	Data.connect("set_jump_marker", Callable(self, "_set_jump_marker"))
	# データを初期化
	bg_image.set_texture(null)
	left_image.set_texture(null)
	right_image.set_texture(null)
	center_image.set_texture(null)
	left_emote.set_texture(null)
	right_emote.set_texture(null)
	center_emote.set_texture(null)
	all_mask.show()
	page_arrow.hide()
	credit_roll.hide()
	fade_mask.hide()
	skip_button.hide()
	next_button_image.hide()
	input_window.show()
	next_button.show()
	skip_marker = ""
	line_number = 0
	current_speaker_name = ""
	enable_text_animation_smoothing = true
	story_label.set_text("")
	### debug mode
	if Data.enable_storylib_debug_mode:
		debug_menu.show()
		debug_line_edit.set_text(str(line_number))
	else:
		debug_menu.hide()
	# 処理スタート
	anim.play("fadein_window")
	await get_tree().create_timer(1.0)
	process_line()

# 辞書から対応マーカーを代入して辞書をリセットし、行処理を継続する
func _set_jump_marker(choice_number):
	jump_marker = Data.storylib_choices[choice_number]["label_name"]
	set_jump_marker(jump_marker)
	Data.storylib_choices = {}
	next_button.show()
	process_line()

# 一行ごとに内容を処理する
func process_line():
	# スキップボタンの可視化判定
	if enabled_skip_button and skip_marker != null and skip_marker != "":
		skip_button.show()
	
	skip_button.disabled = true
	all_mask.show()
	next_button_image.hide()
	page_arrow.hide()
	var words: String
	var file_path = Data.events_directory + "/" + Data.current_storylib_text + ".txt"
	# 処理を行う行を検出し、wordsに代入する
	if FileAccess.file_exists(file_path):
		var file = FileAccess.open(file_path, FileAccess.READ)
		# ファイルが存在している場合、処理を続ける
		if file != null:
			var file_text = file.get_as_text()
			# テキストが空ではない場合、処理を続ける
			if file_text != "":
				var lines = file_text.split("\n")
				var page_index = line_number
				
				if page_index >= 0 and page_index < lines.size():
					# 無限再帰と参照エラー回避のため、テキスト内の整数や少数をstringに変換し、余白を削除する
					words = str(lines[page_index]).strip_edges()
				else:
					push_error("StoryLib Error: ジャンプマーカ―" + jump_marker + "に無限再帰のリスクを検出したため処理を停止しました。", self)
					return
		else:
			push_error("StoryLib Error: " + Data.current_storylib_text + "ファイルが存在しないため、処理を停止しました。", self)
			return
		# 開けたら閉じる
		file.close()
	
	# 行の内容を識別する処理
	# 順番に抜けがあったりコメント行の場合、自動的にその行を飛ばす
	if words.begins_with("#") or words.begins_with("//") or words == null or words == "":
		return next()
	# ジャンプマーカーが有効化されていて、jump_markerが空ではない場合
	elif jump_marker_is_valid and (jump_marker != null or jump_marker != ""):
		# @を省いてマーカー名が一致した場合、ジャンプマーカーを無効にして次の行から開始する
		if words.begins_with("@") and words.lstrip("@") == jump_marker:
			jump_marker = ""
			jump_marker_is_valid = false
		
		return next()
	# 移動先のテキストの@が通常テキストとして認識される現象の対策
	elif words.begins_with("@") and !jump_marker_is_valid:
		return next()
	# コマンド([)を識別する
	elif words.begins_with("[") and !not_command:
		var command = words.lstrip("[").split(" ", true, 1)
		# 選択肢を出現させる 記述例: [choice3 label1, marker1, label2, marker2, label3, marker3
		if command[0].begins_with("choice"):
			# 要素を指定の区切り文字で分割する
			var split_command = command[1].split(", ")
			# 要素が偶数だった場合、キーと要素のペアごとに処理を行う
			if split_command.size() % 2 == 0:
				for i in range(split_command.size() / 2):
					# 辞書に格納していく
					Data.storylib_choices[i] = {}
					Data.storylib_choices[i]["title"] = split_command[i * 2]
					Data.storylib_choices[i]["label_name"] = split_command[i * 2 + 1]
				# 選択肢画面を出現させる
				Data.emit_signal("node", "add", "ChoicesPanel")
				Data.emit_signal("set_choise_buttons")
				skip_button.hide()
				return
			else:
				push_error("StoryLib Error: choiseコマンドの要素数が奇数です。必ずタイトルとペアになるように記述してください。", self)
		# フェードインとフェードアウトの処理
		elif command[0].begins_with("fadein_") or command[0].begins_with("fadeout_"):
			match command[0]:
				# 一括処理の判定
				"fadeout_emotes":
					anim_left.play("fadeout_left_emote")
					anim_right.play("fadeout_right_emote")
					anim_center.play("fadeout_center_emote")
					await get_tree().create_timer(fadein_timer).timeout
				"fadeout_pd":
					anim_left.play("fadeout_left")
					anim_right.play("fadeout_right")
					anim_center.play("fadeout_center")
					await get_tree().create_timer(fadein_timer).timeout
				# BGMの判定
				"fadein_bgm":
					Data.emit_signal("audio", "fadein", null, true)
				"fadeout_bgm":
					Data.emit_signal("audio", "fadeout", null, true)
				_: # それ以外の場合
					var command_position = command[0].split("_", true, 1)
					var position = command_position[1]
					var main_node = null
					var mirror_node = null
					var anim_node
					# ポジションに適切なノードを割り当てる
					match position:
						"bg":
							main_node = bg_image
							mirror_node = bg_mirror
							anim_node = anim
						"left":
							main_node = left_image
							mirror_node = left_mirror
							anim_node = anim_left
						"right":
							main_node = right_image
							mirror_node = right_mirror
							anim_node= anim_right
						"center":
							main_node = center_image
							mirror_node = center_mirror
							anim_node = anim_center
						"front":
							main_node = front_image
							mirror_node = front_mirror
							anim_node = anim
						"left_emote":
							main_node = left_emote
							mirror_node = left_emote_mirror
							anim_node = anim_left
						"right_emote":
							main_node = right_emote
							mirror_node = right_emote_mirror
							anim_node= anim_right
						"center_emote":
							main_node = center_emote
							mirror_node = center_emote_mirror
							anim_node = anim_center
						"all":
							anim_node = anim
						"window":
							anim_node = anim
						_:
							push_error("StoryLib Error: 登録されていないポジションです。", self)
					
					var preloaded_texture
					# command[1]が存在している場合、画像をプリロードする
					if len(command) >= 2:
						preloaded_texture = Data.emit_signal("get_preloaded_instance", command[1])
					# フェードイン用にミラーを透明化し、画像を挿入する
					if command[0].begins_with("fadein_"):
						if mirror_node != null:
							mirror_node.modulate = Color(255, 255, 255, 0)
							Data.emit_signal("set_image", mirror_node, null, command[1])
					elif command[0] == "fadeout_all":
						fade_mask.show()
					# フェードアニメーション開始
					anim_node.play(command[0])
					await get_tree().create_timer(fadein_timer).timeout
					# 後処理
					if command[0].begins_with("fadein_"):
						if position == "all":
							fade_mask.hide()
						elif position != "window":
							Data.emit_signal("set_image", main_node, mirror_node, command[1])
			
			return next()
		# 複数階層に対応したifとif_notの処理: [if second_nest>second_bool, false, that_marker
		elif command[0].begins_with("if"):
			var field = command[1].split(", ", true, 2)
			var directory_name = field[0].split(">", true, -1) # 階層判定用（-1で無制限に分割）
			var reference_data = Data.current_temporary_slot_data
			var current_data = reference_data
			# 上記を分割した数だけ繰り返す
			for i in range(directory_name.size()):
				var key = directory_name[i]
				
				if current_data.has(key):
					current_data = current_data[key]
				else:
					break
				print(i, " 比較: ", current_data, " : ", field[1])
				# 比較データをStringに変換し、キーの要素が指定通りであれば処理を行う
				if command[0] == "if" and str(current_data) == str(field[1]):
					set_jump_marker(field[2])
				# if_not: 一応動作はするが要検証のためコメントアウト
#				elif command[0] == "if_not" and str(current_data) != str(field[1]):
#					set_jump_marker(field[2])
			
			return next()
		else: # 上記以外のコマンド処理
			match command[0]:
				"wait": # x秒待機する
					await get_tree().create_timer(1.0 * float(command[1])).timeout
				# 画像系 #############################################################
				"bg": # 背景
					Data.emit_signal("set_image", bg_image, bg_mirror, command[1])
				"left": # 左
					Data.emit_signal("set_image", left_image, left_mirror, command[1])
				"right": # 右
					Data.emit_signal("set_image", right_image, right_mirror, command[1])
				"center": # 中央
					Data.emit_signal("set_image", center_image, center_mirror, command[1])
				"front": # 前全面
					Data.emit_signal("set_image", front_image, front_mirror, command[1])
				"left_emote": # 左エモート
					Data.emit_signal("set_image", left_emote, left_emote_mirror, command[1])
				"right_emote": # 右エモート
					Data.emit_signal("set_image", right_emote, right_emote_mirror, command[1])
				"center_emote": # 中央エモート
					Data.emit_signal("set_image", center_emote, center_emote_mirror, command[1])
				# 画像を横軸に反転させる
				"h_bg":
					flip_image(bg_image, bg_mirror, true)
				"h_left":
					flip_image(left_image, left_mirror, true)
				"h_right":
					flip_image(right_image, right_mirror, true)
				"h_center":
					flip_image(center_image, center_mirror, true)
				"h_front":
					flip_image(front_image, front_mirror, true)
				"h_left_emote":
					flip_image(left_emote, left_emote_mirror, true)
				"h_right_emote":
					flip_image(right_emote, right_emote_mirror, true)
				"h_center_emote":
					flip_image(center_emote, center_emote_mirror, true)
				# 画像を縦軸に反転させる
				"v_bg":
					flip_image(bg_image, bg_mirror, false)
				"v_left":
					flip_image(left_image, left_mirror, false)
				"v_right":
					flip_image(right_image, right_mirror, false)
				"v_center":
					flip_image(center_image, center_mirror, false)
				"v_front":
					flip_image(front_image, front_mirror, false)
				"v_left_emote":
					flip_image(left_emote, left_emote_mirror, false)
				"v_right_emote":
					flip_image(right_emote, right_emote_mirror, false)
				"v_center_emote":
					flip_image(center_emote, center_emote_mirror, false)
				# 画像を削除する
				"c_left":
					reset_image(left_image)
				"c_right":
					reset_image(right_image)
				"c_center":
					reset_image(center_image)
				"c_front":
					reset_image(front_image)
				"c_left_emote":
					reset_image(left_emote)
				"c_right_emote":
					reset_image(right_emote)
				"c_center_emote":
					reset_image(center_emote)
				"c_emotes": # エモート一括削除
					reset_image(left_emote)
					reset_image(right_emote)
					reset_image(center_emote)
				"c_pd": # 背景と前全面除き一括削除（以降のバージョンで削除予定のコマンドです）
					reset_image(left_image)
					reset_image(right_image)
					reset_image(center_image)
					reset_image(left_emote)
					reset_image(right_emote)
					reset_image(center_emote)
				"c_pds": # 背景と前全面除き一括削除
					reset_image(left_image)
					reset_image(right_image)
					reset_image(center_image)
					reset_image(left_emote)
					reset_image(right_emote)
					reset_image(center_emote)
				"c_all": # 背景除き一括削除
					reset_image(left_image)
					reset_image(right_image)
					reset_image(center_image)
					reset_image(front_image)
					reset_image(left_emote)
					reset_image(right_emote)
					reset_image(center_emote)
				# 可視化
				"show_bg":
					set_visibility_image(bg_image, bg_mirror, true)
				"show_left":
					set_visibility_image(left_image, left_mirror, true)
				"show_right":
					set_visibility_image(right_image, right_mirror, true)
				"show_center":
					set_visibility_image(center_image, center_mirror, true)
				"show_front":
					set_visibility_image(front_image, front_mirror, true)
				"show_left_emote":
					set_visibility_image(left_emote, left_emote_mirror, true)
				"show_right_emote":
					set_visibility_image(right_emote, right_emote_mirror, true)
				"show_center_emote":
					set_visibility_image(center_emote, center_emote_mirror, true)
				# 非可視化
				"hide_bg":
					set_visibility_image(bg_image, bg_mirror, false)
				"hide_left":
					set_visibility_image(left_image, left_mirror, false)
				"hide_right":
					set_visibility_image(right_image, right_mirror, false)
				"hide_center":
					set_visibility_image(center_image, center_mirror, false)
				"hide_front":
					set_visibility_image(front_image, front_mirror, false)
				"hide_left_emote":
					set_visibility_image(left_emote, left_emote_mirror, false)
				"hide_right_emote":
					set_visibility_image(right_emote, right_emote_mirror, false)
				"hide_center_emote":
					set_visibility_image(center_emote, center_emote_mirror, false)
				# 画像系ここまで #########################################################
				"play_bgm": # BGMを再生する
					Data.emit_signal("audio", "play", command[1], true)
				"pause_bgm": # BGMを一時停止する
					Data.emit_signal("audio", "pause", null, true)
				"resume_bgm": # BGMを途中から再開する
					Data.emit_signal("audio", "resume", null, true)
				"stop_bgm": # BGMを止める
					Data.emit_signal("audio", "stop", null, true)
				"play_se": # SEを再生する
					Data.emit_signal("audio", "play", command[1], false)
				"flash": # 0.5秒のフラッシュ
					anim.play("flash")
					await get_tree().create_timer(0.5).timeout
#				# タイムライン型のリアルタイムテキスト機能
#				# 直前のテキストをインスタンス可し、次のテキストと共に整列させて表示する
#				# これは試験的な内容であり、参考のために記述しています
#				"ns": # Next Speech
#					# 連番のキー名を決定する
#					var key = len(realtime_texts)
#					realtime_texts[key] = current_text
#					#current_text = ""
#					story_label.set_text("")
#					print(realtime_texts)
				"speaker": # 発言者名をセットする
					current_speaker_name = conv_control_caracter(command[1])
				"c": # 発言者名とテキストラベルとリアルタイムテキストをクリアする
					current_speaker_name = ""
					story_label.set_text("")
					realtime_texts = {}
				"set": # 指定キーの要素に任意の値をセットする
					var field = command[1].split(", ", true, 1)
					Data.emit_signal("manage_save_data", "update " + field[0] + ", " + field[1], Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
				"save": # 現在のセーブスロットを保存する
					Data.emit_signal("manage_save_data", "update location, " + Data.current_storylib_text, Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
					Data.emit_signal("manage_save_data", "save_per_slot " + Data.current_save_slot, Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
				"load": # 現在のセーブスロットをロードする
					Data.emit_signal("manage_save_data", "load_per_slot " + Data.current_save_slot, Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
					Data.current_storylib_text = Data.temporary_saves[Data.current_save_slot]["location"]
					_ready()
					return
				"open_save_panel": # セーブパネルを開く
					Data.emit_signal("manage_save_data", "update location, " + Data.current_storylib_text, Data.save_file_name, Data.save_file_extension, Data.use_encrypt)
					Data.emit_signal("node", "add", "SavesPanel")
					Data.emit_signal("set_save_slot_buttons", "save")
				"goto": # 指定マーカーへジャンプ
					set_jump_marker(command[1])
				"next": # 指定テキストファイルの最初にジャンプ
					Data.current_storylib_text = command[1]
					line_number = 0
					skip_marker = ""
				"next_scene": # StoryLibを終了して次のシーンへ
					Data.emit_signal("node", "add", command[1])
					queue_free()
					return
				"end": # StoryLibを終了する
					queue_free()
					return
				"enabled_skip_button":
					enabled_skip_button = true
				"disabled_skip_button":
					enabled_skip_button = false
				"set_skip_marker": # スキップ用のマーカーを指定する
					skip_marker = command[1]
				# テキストのスムーズ表示を有効にする
				"enable_smooth_text":
					enable_text_animation_smoothing = true
				# テキストのスムーズ表示を無効にする
				"disable_smooth_text":
					enable_text_animation_smoothing = false
				# コンソールから全ての一時セーブデータを見る
				"show_saves":
					print(Data.temporary_saves)
				# コンソールから現行スロットの一時セーブデータを見る
				"show_current_slot":
					print(Data.current_temporary_slot_data)
				# クレジットロールを再生する手順
				# ①全文はCreditRoll>RichTextLabelに記述してください。
				# ②記述済みのRichTextLabelのTransform>Size>yの値を、
				# AnimCreditRollノードのアニメーションエディタ（当該ノードをクリックして画面下のアニメーションを選択）
				# に登録されている、startアニメーションの、左から2番目のキーのValue>yに負の値として(頭に-をつける)入力します。
				# ※キーが見えない場合は、横スクロールを動かしたり、右下のズーム機能を活用してください。
				# ③startの再生時間をエディタの右側で指定し、エンドロールを終わらせたい時間の位置に2番目のキーを移動させます。
				"play_credit_roll":
#					# 以下はエンジンによる対応待ちの処理のためコメントアウト
#					# 再生時間を30秒に指定する例: [play_credit_roll 30
#					anim_credit_roll.get_animation("start").set_length(float(command[1]))
					
					debug_menu.hide()
					skip_button.hide()
					anim.play("fadeout_window")
					await get_tree().create_timer(1.0).timeout
					anim_credit_roll.play("stand_by")
					credit_roll.show()
					anim.play("fadein_window")
					await get_tree().create_timer(2.0).timeout
					anim_credit_roll.play("start")
					# 再生が完了するまで次の処理を待つ
					await get_tree().create_timer(anim_credit_roll.get_current_animation_length()).timeout
				# クレジットロールを終了する
				"end_credit_roll":
					anim.play("fadeout_window")
					await get_tree().create_timer(1.0).timeout
					credit_roll.hide()
					anim.play("fadein_window")
					
					if enabled_skip_button:
						skip_button.show()
					
					if Data.enable_storylib_debug_mode:
						debug_menu.show()
				_:
					# 登録されていないコマンドを検出した場合、文字列として処理する
					not_command = true
					return await process_line()
			
			return next()
	# テキストを表示する
	else:
		# コマンド判定を初期化
		if not_command:
			not_command = false
		# 「[」スキップ用文字列をトリムする
		if words.begins_with("::"):
			words = words.substr(2)
		# 発言者名がセットされていてまだ表示されていない場合、それを表示する
		if current_speaker_name != null and current_speaker_name != "":
			var text = story_label.get_text()
			var expectedPrefix = current_speaker_name + ":\n"
			
			if text.substr(0, expectedPrefix.length()) != expectedPrefix:
				story_label.set_text(expectedPrefix + text)
		# テキストに制御文字が含まれている場合、それを変換する
		words = conv_control_caracter(words)
		# リアルタイムテキストが空ではない場合
		if realtime_texts.size() > 0:
			# テキストを格納する変数
			var texts = ""
			# キーのリストを取得し、昇順にソートする
			var keys = realtime_texts.keys()
			keys.sort()
			# 連番キーの数字が若いほうから順番に処理する
			for i in keys:
				# テキストを追加
				texts += realtime_texts[i] + "\n"
#			# 連番キーの数字が多いほうから順番に処理する（参考のために追記。このままコメントアウトしておいてください）
#			while keys.size() > 0:
#				var max_key = keys.max()
#				keys.erase(max_key)
#				# 要素を取得して story_text に追加
#				texts += realtime_texts[max_key] + "\n"
			# ラベルにテキストを追加する
			story_label.set_text(story_label.get_text() + texts)
		# テキスト表示をスムーズに
		if enable_text_animation_smoothing:
			var character_count = 0
			# これまでのテキストを一時保存する
			var temp_words = story_label.get_text()
			#.txtで読み込んだ際に\nを正しく認識させる
			var trim_words = words.c_unescape()
			# trim_wordsを一度だけ呼び出して結果を変数に保存する
			var unescaped_words = trim_words
			# character_countがtrim_words以上になるまで繰り返す
			while character_count <= trim_words.length():
				var character = trim_words.left(character_count)
				var all_words = temp_words + character
				character_count += 1
				story_label.set_text(all_words)
				# 以下の2つのアニメーション方法は、どちらかをコメントアウトしてお使いください
				# アニメーション方法1: 一文字ずつ0.02秒間隔で表示させる
				await get_tree().create_timer(0.02).timeout
				# アニメーション方法2: フレームレートに合わせてテキストを更新する
				#await get_tree().create_timer(get_process_delta_time()).timeout
		else:
			story_label.set_text(story_label.get_text() + words.c_unescape())
#		# スクロールを最下段に移動する（試験的内容のためコメントアウト）
#		story_label.scroll_to_line(story_label.get_line_count())
		# 現在のテキストをリアルタイム表示用に保存する
		current_text = words
		
		page_arrow.show()
		anim.play("page_arrow_on")
		
		if enabled_skip_button:
			skip_button.disabled = false
		
		all_mask.hide()
		next_button_image.show()
		return add_line()

func set_jump_marker(field):
	jump_marker = field
	jump_marker_is_valid = true
	line_number = 0

func next():
	add_line()
	process_line()

func pressed_skip_button():
	if skip_marker != null and skip_marker != "":
		set_jump_marker(skip_marker)
		process_line()
		skip_marker = ""
		# 押されたスキップボタンを非表示にする
		skip_button.hide()
		# タッチで次へを表示する
		next_button.show()

# 画像情報のリセット
func reset_image(texture):
	texture.set_texture(null)
	# 透過率リセット
	texture.modulate = Color(1, 1, 1, 1)
	# 向きリセット
	if texture.is_flipped_h():
		texture.set_flip_h(false)
	
	if texture.is_flipped_v():
		texture.set_flip_v(false)

# 画像を反転する
func flip_image(main_node, mirror_node, is_flip_h):
	if is_flip_h:
		if main_node.is_flipped_h():
			mirror_node.set_flip_h(false)
			main_node.set_flip_h(false)
		else:
			mirror_node.set_flip_h(true)
			main_node.set_flip_h(true)
	else:
		if main_node.is_flipped_v():
			mirror_node.set_flip_h(false)
			main_node.set_flip_v(false)
		else:
			mirror_node.set_flip_h(true)
			main_node.set_flip_v(true)

# 画像を表示/非表示
func set_visibility_image(main_node, mirror_node, is_visibility):
	if is_visibility:
		main_node.modulate = Color(1, 1, 1, 1)
		mirror_node.modulate = Color(1, 1, 1, 1)
		main_node.show()
		mirror_node.show()
	else:
		main_node.hide()
		mirror_node.hide()

func add_line():
	line_number = line_number + 1
	### debug mode
	debug_line_edit.set_text(str(line_number))

func debug_utility():
	if !debug_page_debugger.visible:
		debug_page_debugger.show()
	else:
		debug_page_debugger.hide()

func debug_back_page():
	if line_number > 1:
		line_number = line_number - 1
		debug_line_edit.set_text(str(line_number))

func debug_next_page():
	add_line()
	debug_line_edit.set_text(str(line_number))

func debug_page_update():
	var debug_number = debug_line_edit.get_text()
	# debug_numberの全ての文字列が0～9である場合
	if is_numeric(debug_number):
		debug_number = int(debug_number)
		
		if debug_number <= line_number:
			line_number = debug_number - 1
			process_line()
		else:
			push_error("StoryLib Error: 指定された行数はテキストに記述されている範囲を超えています。", self)
	else:
		push_error("StoryLib Error: 行数は半角数字で入力してください。", self)

# 指定の全ての文字列が0～9である場合
func is_numeric(text: String) -> bool:
	var pattern = "^[0-9]+$"
	var regex = RegEx.new()
	regex.compile(pattern)
	return regex.search(text) != null

# 制御文字が含まれている場合、適切な処理をして出力する
# 注意: 現在の仕様では、1行に複数の制御文字を入れると、2番目の制御文字の変換が機能しないことがあります
func conv_control_caracter(text):
	# 制御文字の変換処理
	var control_pattern = ":v:\\[([^\\]]+)\\]"
	var regex = RegEx.new()
	var error = regex.compile(control_pattern)
	var control_character_list = Data.get_file_text(Data.data_templates_directory + Data.control_char_file_name + ".json", false)
	
	if error != OK:
		print("正規表現のコンパイルエラー:", error)
	else:
		var regex_match = regex.search(text)
		
		while regex_match:
			var variable = regex_match.get_string() # マッチした部分全体のテキストを取得する方法
			variable = variable.replace(":v:[", "")
			variable = variable.substr(0, variable.length() - 1)
			
			if control_character_list.has(variable):
				var value = control_character_list[variable]
				text = text.replace(regex_match.get_string(), value)
			
			regex_match = regex.search(text, regex_match.get_end())
	
	return text
